@@ -1,18 +1,7 @@
 #=======================================================================
-#    ____  ____  _____              _    ____ ___   ____
-#   |  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
-#   | |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
-#   |  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
-#   |_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
-#
-#   A Perl Module Chain to faciliate the Creation and Modification
-#   of High-Quality "Portable Document Format (PDF)" Files.
-#
-#=======================================================================
 #
 #   THIS IS A REUSED PERL MODULE, FOR PROPER LICENCING TERMS SEE BELOW:
 #
-#
 #   Copyright Martin Hosken <Martin_Hosken@sil.org>
 #
 #   No warranty or expression of effectiveness, least of all regarding
@@ -20,20 +9,19 @@
 #
 #   This specific module is licensed under the Perl Artistic License.
 #
-#
-#   $Id: Dict.pm,v 1.9 2005/01/21 01:27:18 fredo Exp $
-#
 #=======================================================================
 package PDF::API2::Basic::PDF::Dict;
 
+our $VERSION = '2.019';
+
+use base 'PDF::API2::Basic::PDF::Objind';
+
 use strict;
-use vars qw(@ISA $mincache $tempbase $cr);
 no warnings qw[ deprecated recursion uninitialized ];
 
-use PDF::API2::Basic::PDF::Objind;
-@ISA = qw(PDF::API2::Basic::PDF::Objind);
-
-$cr = '(?:\015|\012|(?:\015\012))';
+our $mincache;
+our $tempbase;
+our $cr = '(?:\015|\012|(?:\015\012))';
 
 use PDF::API2::Basic::PDF::Filter;
 use PDF::API2::Basic::PDF::Name;
@@ -54,6 +42,8 @@ PDF::API2::Basic::PDF::Dict - PDF Dictionaries and Streams. Inherits from L<PDF:
 There are various special instance variables which are used to look after,
 particularly, streams. Each begins with a space:
 
+=over
+
 =item stream
 
 Holds the stream contents for output
@@ -68,6 +58,8 @@ not the same as a PDF file stream. The data is stored in its unfiltered form.
 If both ' stream' and ' streamfile' are empty, this indicates where in the
 source PDF the stream starts.
 
+=back
+
 =head1 METHODS
 
 =cut
@@ -230,109 +222,6 @@ sub outobjdeep
 
 }
 
-sub outxmldeep
-{
-    my ($self, $fh, $pdf, %opts) = @_;
-    my ($key, $val, $f, @filts);
-    my ($loc, $str, %specs, $len);
-
-    $opts{-xmlfh}->print("<Dict>\n");
-    foreach ('Type', 'Subtype')
-    {
-        $specs{$_} = 1;
-        if (defined $self->{$_})
-        {
-            $opts{-xmlfh}->print("<Key id=\"$_\">");
-            $self->{$_}->outxml($fh, $pdf, %opts);
-            $opts{-xmlfh}->print("</Key>\n");
-        }
-    }
-    while (($key, $val) = each %{$self})
-    {
-        next if ($key =~ m/^[\s\-]/o || $specs{$key});
-        next if ($val eq '');
-    $key = PDF::API2::Basic::PDF::Name::string_to_name ($key, $pdf);
-        $opts{-xmlfh}->print("<Key id=\"$key\">");
-        $val->outxml($fh, $pdf, %opts);
-        $opts{-xmlfh}->print("</Key>\n");
-    }
-    $opts{-xmlfh}->print("</Dict>\n");
-
-#now handle the stream (if any)
-    if (defined $self->{' streamloc'} && !defined $self->{' stream'})
-    {                                   # read a stream if infile
-        $loc = $fh->tell;
-        $self->read_stream;
-        $fh->seek($loc, 0);
-    }
-
-    if (!$self->{' nofilt'}
-            && (defined $self->{' stream'} || defined $self->{' streamfile'})
-            && defined $self->{'Filter'})
-    {
-        my ($hasflate) = -1;
-        my ($temp, $i, $temp1);
-
-        for ($i = 0; $i < scalar @{$self->{'Filter'}{' val'}}; $i++)
-        {
-            $temp = $self->{'Filter'}{' val'}[$i]->val;
-            if ($temp eq 'LZWDecode')               # hack to get around LZW patent
-            {
-                if ($hasflate < -1)
-                {
-                    $hasflate = $i;
-                    next;
-                }
-                $temp = 'FlateDecode';
-                $self->{'Filter'}{' val'}[$i]{'val'} = $temp;      # !!!
-            } elsif ($temp eq 'FlateDecode')
-            { $hasflate = -2; }
-            $temp1 = "PDF::API2::Basic::PDF::$temp";
-            push (@filts, $temp1->new);
-        }
-        splice(@{$self->{'Filter'}{' val'}}, $hasflate, 1) if ($hasflate > -1);
-    }
-
-    if (defined $self->{' stream'})
-    {
-        $opts{-xmlfh}->print("<Stream>\n");
-        $loc = $fh->tell;
-        $str = $self->{' stream'};
-        if ($self->{' nofilt'})
-        {
-            foreach $f (@filts)
-            { $str = $f->infilt($str, 1); }
-        }
-        $opts{-xmlfh}->print($str);
-        $opts{-xmlfh}->print("</Stream>\n");
-    } elsif (defined $self->{' streamfile'})
-    {
-        open(DICTFH, $self->{' streamfile'}) || die "Unable to open $self->{' streamfile'}";
-        binmode(DICTFH.':raw');
-        $opts{-xmlfh}->print("<Stream>\n");
-        while (read(DICTFH, $str, 4096))
-        {
-            if ($self->{' nofilt'})
-            {
-                foreach $f (@filts)
-                { $str = $f->infilt($str, 0); }
-            }
-        $opts{-xmlfh}->print($str);
-        }
-        close(DICTFH);
-        if ($self->{' nofilt'})
-        {
-            $str = '';
-            foreach $f (@filts)
-            { $str = $f->infilt($str, 1); }
-        $opts{-xmlfh}->print($str);
-        }
-
-        $opts{-xmlfh}->print("</Stream>\n");
-    }
-}
-
-
 =head2 $d->read_stream($force_memory)
 
 Reads in a stream from a PDF file. If the stream is greater than
@@ -410,5 +299,4 @@ Returns the dictionary, which is itself.
 sub val
 { $_[0]; }
 
-
-
+1;
