@@ -1,18 +1,7 @@
 #=======================================================================
-#    ____  ____  _____              _    ____ ___   ____
-#   |  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
-#   | |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
-#   |  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
-#   |_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
-#
-#   A Perl Module Chain to faciliate the Creation and Modification
-#   of High-Quality "Portable Document Format (PDF)" Files.
-#
-#=======================================================================
 #
 #   THIS IS A REUSED PERL MODULE, FOR PROPER LICENCING TERMS SEE BELOW:
 #
-#
 #   Copyright Martin Hosken <Martin_Hosken@sil.org>
 #
 #   Modified for PDF::API2 by Alfred Reibenschuh <alfredreibenschuh@gmx.net>
@@ -22,12 +11,11 @@
 #
 #   This specific module is licensed under the Perl Artistic License.
 #
-#
-#   $Id: File.pm,v 2.0 2005/11/16 02:16:00 areibens Exp $
-#
 #=======================================================================
 package PDF::API2::Basic::PDF::File;
 
+our $VERSION = '2.019';
+
 =head1 NAME
 
 PDF::API2::Basic::PDF::File - Holds the trailers and cross-reference tables for a PDF file
@@ -155,7 +143,7 @@ BEGIN
 {
     use strict;
     no strict "refs";
-    use vars qw($cr $irreg_char $reg_char $ws_char $delim_char %types $VERSION);
+    use vars qw($cr $irreg_char $reg_char $ws_char $delim_char %types);
     # no warnings qw(uninitialized);
     use strict;
 
@@ -201,9 +189,6 @@ use PDF::API2::Basic::PDF::Null;
 
 no warnings qw[ deprecated recursion uninitialized ];
 
-    # $Revisioning: 0.40_11 on Mon Oct  6 18:39:40 2003 $
-    ( $VERSION ) = '0.40_10';
-
 #IMPORTED INTO PDF::API2
 
 #$VERSION = "0.23";      # MJPH  14-AUG-2002     Fix MANIFEST
@@ -274,56 +259,6 @@ sub open
     my ($self, $buf, $xpos, $end, $tdict, $k);
     my ($fh);
 
-    $self = $class->_new;
-    if (ref $fname)
-    {
-        $self->{' INFILE'} = $fname;
-        $fh = $fname;
-    }
-    else
-    {
-        $fh = IO::File->new(($update ? "+" : "") . "<$fname") || return undef;
-        $self->{' INFILE'} = $fh;
-    }
-
-    binmode($fh,':raw');
-    if ($update)
-    {
-        $self->{' update'} = 1;
-        $self->{' OUTFILE'} = $fh;
-        $self->{' fname'} = $fname;
-    }
-    $fh->read($buf, 255);
-    if ($buf !~ m/^\%pdf\-1\.(\d)\s*/moi)
-    { die "$fname not a PDF file version 1.x"; }
-    else
-    { $self->{' Version'} = $1; }
-
-    $fh->seek(0, 2);            # go to end of file
-    $end = $fh->tell();
-    $self->{' epos'} = $end;
-#    if (!$fh->seek(($end > 1024 ? $end - 1024 : 0, 0)))
-#      { die "Seek failed when reading PDF file $fname"; }
-#    $fh->read($buf, 1024);
-    if (!$fh->seek(($end > 64 ? $end - 64 : 0, 0)))
-      { die "Seek failed when reading PDF file $fname"; }
-    $fh->read($buf, 64);
-    if ($buf !~ m/startxref$cr([0-9]+)$cr\%\%eof.*?$/oi)
-    { die "Malformed PDF file $fname"; }
-    $xpos = $1;
-
-    $tdict = $self->readxrtr($xpos, $self);
-    foreach $k (keys %{$tdict})
-    { $self->{$k} = $tdict->{$k}; }
-    return $self;
-}
-
-sub open_swallowed
-{
-    my ($class, $fname, $update) = @_;
-    my ($self, $buf, $xpos, $end, $tdict, $k);
-    my ($fh);
-
     $self = $class->_new;
     if (ref $fname)
     {
@@ -340,7 +275,6 @@ sub open_swallowed
         die "File '$fname' does not exist !" unless(-f $fname);
         $fh = IO::File->new(($update ? "+" : "") . "<$fname") || return undef;
         $self->{' INFILE'} = $fh;
-        binmode($fh,':raw');
         if ($update)
         {
             $self->{' update'} = 1;
@@ -348,6 +282,7 @@ sub open_swallowed
             $self->{' fname'} = $fname;
         }
     }
+    binmode($fh,':raw');
     $fh->seek(0, 0);            # go to start of file
     $fh->read($buf, 255);
     if ($buf !~ m/^\%PDF\-1\.\d+\s*$cr/mo)
@@ -356,11 +291,16 @@ sub open_swallowed
     $fh->seek(0, 2);            # go to end of file
     $end = $fh->tell();
     $self->{' epos'} = $end;
-#    $fh->seek($end - 1024, 0);
-#    $fh->read($buf, 1024);
-    $fh->seek($end - 64, 0);
-    $fh->read($buf, 64);
-    if ($buf !~ m/startxref$cr\s*([0-9]+)$cr\%\%eof.*?/oi)
+    foreach my $eoff (1..64)
+    {
+    	$fh->seek($end - 16*$eoff, 0);
+    	$fh->read($buf, 16*$eoff);
+    	if ($buf =~ m/startxref($cr|\s*)\d+($cr|\s*)\%\%eof.*?/oi)
+    	{
+    		last;
+    	}
+    }
+    if ($buf !~ m/startxref[^\d]+([0-9]+)($cr|\s*)\%\%eof.*?/oi)
     { die "Malformed PDF file $fname"; }
     $xpos = $1;
 
@@ -442,15 +382,9 @@ sub append_file
     # hack to upgrade pdf-version number to support
     # requested features in higher versions that
     # the pdf was originally created.
-    $fh->seek(7,0);
-    $fh->read($buf, 3);
-    $buf=~s/[^\d]+$//g;
-    $ver=$self->{' version'} || 2;
-    if($buf < $ver) {
-    ##  print STDERR "files version was 1.$buf upgraded to 1.$ver.\n";
-        $fh->seek(0,0);
-        $fh->print("%PDF-1.$ver\n");
-    }
+    $ver=$self->{' version'} || 4;
+    $fh->seek(0,0);
+    $fh->print("%PDF-1.$ver\n");
 
     $tdict = PDFDict();
     $tdict->{'Prev'} = PDFNum($self->{' loc'});
@@ -514,7 +448,7 @@ sub create_file
 
     $self->{' OUTFILE'} = $fh;
     $fh->print('%PDF-1.' . ($self->{' version'} || '2') . "\n");
-    $fh->print("%쏢\n");              # and some binary stuff in a comment
+    $fh->print("%\xC7\xEC\xF3\xA2\n");   # and some binary stuff in a comment
     $self;
 }
 
@@ -574,13 +508,11 @@ sub readval
     my ($res, $key, $value, $k);
 
     $str = update($fh, $str);
-    ## updateRef($fh, \$str);
 
     if ($str =~ m/^<</so)
     {
         $str = substr ($str, 2);
         $str = update($fh, $str);
-        ## updateRef($fh, \$str);
         $res = PDFDict();
 
         while ($str !~ m/^>>/o) {
@@ -603,11 +535,9 @@ sub readval
                 $res->{null} = $value;
             }
             $str = update($fh, $str); # thanks gareth.jones@stud.man.ac.uk
-            ## updateRef($fh, \$str); # thanks gareth.jones@stud.man.ac.uk
         }
         $str =~ s/^>>//o;
         $str = update($fh, $str);
-        ## updateRef($fh, \$str);
     # streams can't be followed by a lone carriage-return.
     # fredo: yes they can !!! -- use the MacOS Luke.
         if (($str =~ s/^stream(?:(?:\015\012)|\012|\015)//o)
@@ -629,7 +559,6 @@ sub readval
                 $res->{' stream'} = $value;
                 $res->{' nofilt'} = 1;
                 $str = update($fh, $str, 1);  # tell update we are in-stream and only need an endstream 
-                ## updateRef($fh, \$str, 1);  # tell update we are in-stream and only need an endstream 
                 #$str =~ s/^endstream//o; # we cannot regexpr here since we need the first endstream only
                 # so we do the following:
                 my $wh = index($str,'endstream');
@@ -675,7 +604,6 @@ sub readval
             $res->{' realised'} = 1;
         }
         $str = update($fh, $str);       # thanks to kundrat@kundrat.sk
-        ## updateRef($fh, \$str);       # thanks to kundrat@kundrat.sk
         $str =~ s/^endobj//o;
     } elsif ($str =~ m|^/($reg_char+)|so)        # name
     {
@@ -683,46 +611,54 @@ sub readval
         $value = $1;
         $str =~ s|^/($reg_char+)||so;
         $res = PDF::API2::Basic::PDF::Name->from_pdf($value, $self);
-    } elsif ($str =~ m/^\(/o)  # literal string
+    } 
+    elsif ($str =~ m/^\(/o)  # literal string
     {
         $str =~ s/^\(//o;
-    # We now need to find an unbalanced, unescaped right-paren.
-    # This can't be done with regexps.
-    my ($value) = "";
-    # The current level of nesting, when this reaches 0 we have finished.
-    my ($nested) = 1;
-    while (1) {
-        # Remove everything up to the first (possibly escaped) paren.
-        $str =~ /^((?:[^\\()]|\\[^()])*)(.*)/so;
-        $value .= $1;
-        $str = $2;
-
-        if ($str =~ /^(\\[()])/o) {
-        # An escaped paren.  This would be tricky to do with
-        # the regexp above (it's very difficult to be certain
-        # that all cases are covered so I think it's better to
-        # deal with them explicitly).
-        $str = substr ($str, 2);
-        $value = $value . $1;
-        } elsif ($str =~ /^\)/o) {
-        # Right paren
-        $nested--;
-        $str = substr ($str, 1);
-        if ($nested == 0)
-            { last; }
-        $value = $value . ')';
-        } elsif ($str =~ /^\(/o) {
-        # Left paren
-        $nested++;
-        $str = substr ($str, 1);
-        $value = $value . '(';
-        } else {
-        # No parens, we must read more.  We don't use update
-        # because we don't want to remove whitespace or
-        # comments.
-        $fh->read($str, 255, length($str)) or die "Unterminated string.";
+        # We now need to find an unbalanced, unescaped right-paren.
+        # This can't be done with regexps.
+        my ($value) = "";
+        # The current level of nesting, when this reaches 0 we have finished.
+        my ($nested) = 1;
+        while (1) {
+            # Remove everything up to the first (possibly escaped) paren.
+            $str =~ /^((?:[^\\()]|\\[^()])*)(.*)/so;
+            $value .= $1;
+            $str = $2;
+    
+            if ($str =~ /^(\\[()])/o) 
+            {
+                # An escaped paren.  This would be tricky to do with
+                # the regexp above (it's very difficult to be certain
+                # that all cases are covered so I think it's better to
+                # deal with them explicitly).
+                $str = substr ($str, 2);
+                $value = $value . $1;
+            } 
+            elsif ($str =~ /^\)/o) 
+            {
+                # Right paren
+                $nested--;
+                $str = substr ($str, 1);
+                if ($nested == 0)
+                    { last; }
+                $value = $value . ')';
+            } 
+            elsif ($str =~ /^\(/o) 
+            {
+                # Left paren
+                $nested++;
+                $str = substr ($str, 1);
+                $value = $value . '(';
+            } 
+            else 
+            {
+                # No parens, we must read more.  We don't use update
+                # because we don't want to remove whitespace or
+                # comments.
+                $fh->read($str, 255, length($str)) or die "Unterminated string.";
+            }
         }
-    }
 
         $res = PDF::API2::Basic::PDF::String->from_pdf($value);
     } 
@@ -737,14 +673,12 @@ sub readval
     {
         $str =~ s/^\[//o;
         $str = update($fh, $str);
-        ## updateRef($fh, \$str);
         $res = PDFArray();
         while ($str !~ m/^\]/o)
         {
             ($value, $str) = $self->readval($str, %opts);
             $res->add_elements($value);
             $str = update($fh, $str);   # str might just be exhausted!
-            ## updateRef($fh, \$str);   # str might just be exhausted!
         }
         $str =~ s/^\]//o;
     } 
@@ -1138,42 +1072,6 @@ sub update
     return $str;
 }
 
-sub updateRef
-{
-    my ($fh, $str, $instream) = @_;
-    my $inlen=16; my $inoff=1;
-    print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
-    if($instream) {
-        # we are inside a (possible binary) stream
-        # so we fetch data till we see an 'endstream'
-        # -- fredo/2004-09-03
-        while ($$str !~ m/endstream/o && !$fh->eof)
-        {
-            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
-            $fh->read($$str, 314, length($$str));
-        }
-    } else {
-        $$str =~ s/^$ws_char*//o;
-        while ($$str !~ m/$cr/o && !$fh->eof && length($$str) < 16000)
-        {
-            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
-            $fh->read($$str, $inlen, length($$str));
-            $$str =~ s/^$ws_char*//so;
-            $inlen+=$inoff;
-        }
-        while ($$str =~ m/^\%/o) # restructured by fredo/2003-03-23
-        {
-            print STDERR "fpos=".tell($fh)." strlen=".length($$str)."\n" if($readDebug);
-            $fh->read($$str, $inlen, length($$str)) while ($$str !~ m/$cr/o && !$fh->eof);
-            $$str =~ s/^\%[^\015\012]+$ws_char*//so; # fixed for reportlab -- fredo
-            $inlen+=$inoff;
-        }
-    }
-
-    return undef;
-}
-
-
 =head2 $objind = $p->test_obj($num, $gen)
 
 Tests the cache to see whether an object reference (which may or may not have
@@ -1226,8 +1124,26 @@ sub readxrtr
     $fh->seek($xpos, 0);
     $fh->read($buf, 22);
     $buf=update($fh,$buf); # fix for broken JAWS xref calculation.
+    
+    ## seams that some products calculate wrong prev entries (short)
+    ## so we seek ahead to find one -- fredo; save for now
+    #while($buf !~ m/^xref$cr/oi && !eof($fh))
+    #{
+    #	$buf =~ s/^(\s+|\S+|.)//oi;
+	#    $buf=update($fh,$buf);
+    #}
+    
     if ($buf !~ m/^xref$cr/oi)
-    { die "Malformed xref in PDF file $self->{' fname'}"; }
+    { 
+    	#if($self == $root)
+    	#{
+    		die "Malformed xref in PDF file $self->{' fname'}";
+    	#}
+    	#else # be tolerant if its not the root object
+    	#{
+    	#	return undef;
+    	#} 
+    }
     $buf =~ s/^xref$cr//oi;
 
     $xlist = {};
@@ -1246,10 +1162,10 @@ sub readxrtr
         }
     }
 
-    if ($buf !~ /^trailer$cr/oi)
+    if ($buf !~ /^\s*trailer\b/oi)
     { die "Malformed trailer in PDF file $self->{' fname'} at " . ($fh->tell - length($buf)); }
 
-    $buf =~ s/^trailer$cr//oi;
+    $buf =~ s/^\s*trailer\b//oi;
 
     ($tdict, $buf) = $self->readval($buf);
     $tdict->{' loc'} = $xpos;
@@ -1257,6 +1173,7 @@ sub readxrtr
     $self->{' maxobj'} = $xmin if $xmin > $self->{' maxobj'};
     $tdict->{' prev'} = $self->readxrtr($tdict->{'Prev'}->val)
                 if (defined $tdict->{'Prev'} && $tdict->{'Prev'}->val != 0);
+    delete $tdict->{' prev'} unless(defined $tdict->{' prev'});
     return $tdict;
 }
 
@@ -1384,81 +1301,6 @@ sub _new
     $self;
 }
 
-sub save_xml
-{
-    my ($self,$fh) = @_;
-    my ($tdict, $t, $buf, $ver);
-
-    $fh->print("<Pdf>\n");
-
-    $tdict = PDFDict();
-    $tdict->{'Prev'} = PDFNum($self->{' loc'}) if ($self->{' loc'});
-    $tdict->{'Size'} = $self->{'Size'} || PDFNum(1);
-    $tdict->{'Info'} = $self->{'Info'};
-    if (defined $self->{' newroot'})
-    { $tdict->{'Root'} = $self->{' newroot'}; }
-    else
-    { $tdict->{'Root'} = $self->{'Root'}; }
-    $tdict->{'Size'} = $self->{'Size'};
-
-    foreach $t (grep ($_ !~ m/^\s/o, keys %$self))
-    { $tdict->{$t} = $self->{$t} unless defined $tdict->{$t}; }
-
-    $self->out_xml($tdict,$fh);
-    $fh->print("</Pdf>\n");
-}
-
-sub ship_xml
-{
-    my ($self, $fh, @objs) = @_;
-    my ($n, $objind, $i, $j);
-    my ($objnum, $objgen);
-
-    @objs = @{$self->{' outlist'}} unless (scalar @objs > 0);
-    foreach $objind (@objs)
-    {
-        next unless $objind->is_obj($self);
-        $j = -1;
-        for ($i = 0; $i < scalar @{$self->{' outlist'}}; $i++)
-        {
-            if ($self->{' outlist'}[$i] eq $objind)
-            {
-                $j = $i;
-                last;
-            }
-        }
-        next if ($j < 0);
-        splice(@{$self->{' outlist'}}, $j, 1);
-        delete $self->{' outlist_cache'}{$objind};
-        next if grep {$_ eq $objind} @{$self->{' free'}};
-
-        ($objnum, $objgen) = @{$self->{' objects'}{$objind->uid}}[0..1];
-        $fh->printf("<Object id=\"%d %d\">\n", $objnum, $objgen);
-        $objind->outxmldeep($fh, $self, 'objnum' => $objnum, 'objgen' => $objgen,-xmlfh=>$fh);
-        $fh->print("</Object>\n");
-
-        # Note that we've output this obj, not forgetting to update the cache
-        # of whats printed.
-        unless (exists $self->{' printed_cache'}{$objind})
-        {
-            push( @{$self->{' printed'}}, $objind );
-            $self->{' printed_cache'}{$objind}++;
-        }
-    }
-    $self;
-}
-
-sub out_xml
-{
-    my ($self, $tdict, $fh) = @_;
-    my ($objind, $j, $i, $iend, @xreflist, $first, $k, $xref, $tloc, @freelist);
-    my (%locs, $size);
-
-    while (@{$self->{' outlist'}})
-    { $self->ship_xml($fh); }
-
-}
-
 1;
 
 =head1 AUTHOR
