@@ -1,56 +1,24 @@
-#=======================================================================
-#    ____  ____  _____              _    ____ ___   ____
-#   |  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
-#   | |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
-#   |  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
-#   |_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
-#
-#   A Perl Module Chain to faciliate the Creation and Modification
-#   of High-Quality "Portable Document Format (PDF)" Files.
-#
-#   Copyright 1999-2005 Alfred Reibenschuh <areibens@cpan.org>.
-#
-#=======================================================================
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the
-#   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-#   Boston, MA 02111-1307, USA.
-#
-#   $Id: BaseFont.pm,v 1.15 2005/03/14 22:01:06 fredo Exp $
-#
-#=======================================================================
 package PDF::API2::Resource::BaseFont;
 
-BEGIN {
+our $VERSION = '2.019';
 
-    use utf8;
-    use Encode qw(:all);
+use base 'PDF::API2::Resource';
 
-    use PDF::API2::Util;
-    use PDF::API2::Basic::PDF::Utils;
-    use PDF::API2::Resource;
-    use Compress::Zlib;
-    
-    use vars qw(@ISA $VERSION);
+use Compress::Zlib;
+use Encode qw(:all);
+use PDF::API2::Basic::PDF::Utils;
+use PDF::API2::Util;
 
-    @ISA = qw( PDF::API2::Resource );
-
-    ( $VERSION ) = '$Revision: 1.15 $' =~ /Revision: (\S+)\s/; # $Date: 2005/03/14 22:01:06 $
-
-}
 no warnings qw[ deprecated recursion uninitialized ];
 
+=head1 NAME
+
+PDF::API2::Resource::BaseFont
+
+=head1 METHODS
+
+=over
+
 =item $font = PDF::API2::Resource::BaseFont->new $pdf, $name
 
 Returns a font resource object.
@@ -169,7 +137,7 @@ sub tounicodemap {
         $cmap.=qq|1 begincodespacerange\n<00> <FF>\nendcodespacerange\n|;
         $cmap.=qq|256 beginbfchar\n|; 
         for(my $j=0; $j<256;$j++) {
-            $cmap.=sprintf(qq|<%02X> <%04X>\n|,$j,$self->uniByMap($j));
+            $cmap.=sprintf(qq|<%02X> <%04X>\n|,$j,$self->uniByEnc($j));
         }
         $cmap.=qq|endbfchar\n|;
     }
@@ -196,7 +164,7 @@ sub tounicodemap {
 
 =head1 FONT-MANAGEMENT RELATED METHODS
 
-=over 4
+=over
 
 =item $name = $font->fontname()
 
@@ -250,7 +218,7 @@ sub iscff { return( $_[0]->data->{iscff} ); }
 
 =head1 TYPOGRAPHY RELATED METHODS
 
-=over 4
+=over
 
 =item ($llx, $lly, $urx, $ury) = $font->fontbbox()
 
@@ -491,7 +459,7 @@ Returns the character by the given unicode of the fonts encoding map.
 
 =cut
 
-sub encByUni { return( $_[0]->data->{u2c}->{$_[1]} || $_[0]->data->{u2e}->{$_[1]} || 0 ); }
+sub encByUni { return( $_[0]->data->{u2e}->{$_[1]} || $_[0]->data->{u2c}->{$_[1]} || 0 ); }
 
 =item $char = $font->mapByGlyph $glyph
 
@@ -545,7 +513,38 @@ Returns the glyphs width.
 
 =cut
 
-sub wxByGlyph { return ( (ref($_[0]->data->{wx}) eq 'HASH' ? $_[0]->data->{wx}->{$_[1]} : $_[0]->data->{wx}->[$_[1]]) || $_[0]->missingwidth || 300 ); }
+sub wxByGlyph 
+{
+    my $self=shift;
+    my $val=shift;
+    my $ret=undef; 
+    if(ref($self->data->{wx}) eq 'HASH')
+    {
+    	$ret=$self->data->{wx}->{$val};
+    	if(!defined($ret))
+    	{
+    		$ret=$self->missingwidth;
+    	}
+    	if(!defined($ret))
+    	{
+    		$ret=300;
+    	}
+    }
+    else
+    {
+    	my $cid=$self->cidByUni(uniByName($val));
+    	$ret=$self->data->{wx}->[$cid];
+    	if(!defined($ret))
+    	{
+    		$ret=$self->missingwidth;
+    	}
+    	if(!defined($ret))
+    	{
+    		$ret=300;
+    	}
+    }
+    return $ret;
+}
 
 =item $width = $font->wxByUni $uni
 
@@ -553,7 +552,22 @@ Returns the unicodes width.
 
 =cut
 
-sub wxByUni { return ( $_[0]->data->{wx}->{$_[0]->glyphByUni($_[1])} || $_[0]->missingwidth || 300  ); }
+sub wxByUni 
+{ 
+    my $self=shift;
+    my $val=shift;
+    my $gid=$self->glyphByUni($val);
+    my $ret=$self->data->{wx}->{$gid}; 
+   	if(!defined($ret))
+   	{
+   		$ret=$self->missingwidth;
+   	}
+   	if(!defined($ret))
+   	{
+   		$ret=300;
+   	}
+    return $ret;
+}
 
 =item $width = $font->wxByEnc $char
 
@@ -561,11 +575,20 @@ Returns the characters width based on the current encoding.
 
 =cut
 
-sub wxByEnc {
+sub wxByEnc 
+{
     my ($self,$e)=@_;
     my $g=$self->glyphByEnc($e);
-    my $w=$self->data->{wx}->{$g} || $self->missingwidth || 300;
-    return ( $w );
+    my $ret=$self->data->{wx}->{$g}; 
+   	if(!defined($ret))
+   	{
+   		$ret=$self->missingwidth;
+   	}
+   	if(!defined($ret))
+   	{
+   		$ret=300;
+   	}
+    return $ret;
 }
 
 =item $width = $font->wxByMap $char
@@ -574,7 +597,21 @@ Returns the characters width based on the fonts default encoding.
 
 =cut
 
-sub wxByMap { return ( $_[0]->data->{wx}->{$_[0]->glyphByMap($_[1])} || $_[0]->missingwidth || 300 ); }
+sub wxByMap
+{
+    my ($self,$m)=@_;
+    my $g=$self->glyphByMap($m);
+    my $ret=$self->data->{wx}->{$g}; 
+   	if(!defined($ret))
+   	{
+   		$ret=$self->missingwidth;
+   	}
+   	if(!defined($ret))
+   	{
+   		$ret=300;
+   	}
+    return $ret;
+}
 
 =item $wd = $font->width $text
 
@@ -590,8 +627,15 @@ sub width {
     if(is_utf8($text)) {
         $text=$self->strByUtf($text)
     }
-    foreach my $n (unpack('C*',$text)) {
+    my $lastglyph='';
+    foreach my $n (unpack('C*',$text)) 
+    {
         $width+=$self->wxByEnc($n);
+        if($self->{-dokern} && ref($self->data->{kern}))
+        {
+            $width+=$self->data->{kern}->{$lastglyph.':'.$self->data->{e2n}->[$n]};
+            $lastglyph=$self->data->{e2n}->[$n];
+        }
     }
     $width/=1000;
     return($width);
@@ -617,7 +661,7 @@ sub width_array {
 
 =head1 STRING METHODS
 
-=over 4
+=over
 
 =item $utf8string = $font->utfByStr $string
 
@@ -651,7 +695,8 @@ Returns a properly formatted representation of $text for use in the PDF.
 
 =cut
 
-sub textByStr {
+sub textByStr 
+{
     my ($self,$text)=@_;
     my $newtext='';
     if(is_utf8($text)) 
@@ -665,22 +710,81 @@ sub textByStr {
     return($newtext);
 }
 
-sub text {
-  my ($self,$text,$size)=@_;
+sub textByStrKern 
+{
+    my ($self,$text)=@_;
+    if($self->{-dokern} && ref($self->data->{kern}))
+    {
+        my $newtext=' ';
+        if(is_utf8($text)) 
+        {
+            $text=$self->strByUtf($text);
+        }
 
-  my $newtext=$self->textByStr($text);
-
-  if(defined $size)
-  {
-    return("($newtext) Tj");
-  }
-  else
-  {
-    return("($newtext)");
-  }
+        my $lastglyph='';
+        my $tBefore=0;
+        foreach my $n (unpack('C*',$text)) 
+        {
+            if(defined $self->data->{kern}->{$lastglyph.':'.$self->data->{e2n}->[$n]})
+            {
+                $newtext.=') ' if($tBefore);
+                $newtext.=sprintf('%i ',-($self->data->{kern}->{$lastglyph.':'.$self->data->{e2n}->[$n]}));
+                $tBefore=0;
+            }
+            $lastglyph=$self->data->{e2n}->[$n];
+            my $t=pack('C',$n);
+            $t=~s/\\/\\\\/go;
+            $t=~s/([\x00-\x1f])/sprintf('\%03lo',ord($1))/ge;
+            $t=~s/([\{\}\[\]\(\)])/\\$1/g;
+            $newtext.='(' if(!$tBefore);
+            $newtext.="$t";
+            $tBefore=1;
+        }
+        $newtext.=') ' if($tBefore);
+        return($newtext);
+    }
+    else
+    {
+        return('('.$self->textByStr($text).')');
+    }
 }
 
-sub isvirtual { return(undef); }
+sub text 
+{
+    my ($self,$text,$size,$ident)=@_;
+
+    my $newtext=$self->textByStr($text);
+
+    if(defined $size && $self->{-dokern})
+    {
+        $newtext=$self->textByStrKern($text);
+        if(defined($ident) && $ident!=0)
+        {
+	        return("[ $ident $newtext ] TJ");
+        }
+        else
+        {
+	        return("[ $newtext ] TJ");
+        }
+    }
+    elsif(defined $size)
+    {
+        if(defined($ident) && $ident!=0)
+        {
+	        return("[ $ident ($newtext) ] TJ");
+        }
+        else
+        {
+	        return("[ ($newtext) ] TJ");
+        }
+    }
+    else
+    {
+        return("($newtext)");
+    }
+}
+
+sub isvirtual { return(0); }
 
 1;
 
@@ -690,55 +794,6 @@ __END__
 
 =head1 AUTHOR
 
-alfred reibenschuh.
-
-=head1 HISTORY
-
-    $Log: BaseFont.pm,v $
-    Revision 1.15  2005/03/14 22:01:06  fredo
-    upd 2005
-
-    Revision 1.14  2005/01/21 10:03:09  fredo
-    added object saver for cmap
-
-    Revision 1.13  2004/12/16 00:30:53  fredo
-    added no warn for recursion
-
-    Revision 1.12  2004/11/24 20:10:55  fredo
-    added virtual font handling
-
-    Revision 1.11  2004/11/22 02:05:32  fredo
-    added pdf-1.5 font param specs
-
-    Revision 1.10  2004/10/26 14:41:37  fredo
-    added panose identification style entry
-
-    Revision 1.9  2004/10/17 03:55:00  fredo
-    simplified ToUnicode associated CMap for single-byte fonts
-
-    Revision 1.8  2004/10/17 03:47:36  fredo
-    fixed inclusion of ToUnicode compatible key and associated CMap
-
-    Revision 1.7  2004/06/15 09:14:41  fredo
-    removed cr+lf
-
-    Revision 1.6  2004/06/07 19:44:36  fredo
-    cleaned out cr+lf for lf
-
-    Revision 1.5  2004/04/20 09:47:34  fredo
-    fixed unicode to font-encoding-vector conversion
-
-    Revision 1.4  2003/12/08 13:05:32  Administrator
-    corrected to proper licencing statement
-
-    Revision 1.3  2003/11/30 17:28:54  Administrator
-    merged into default
-
-    Revision 1.2.2.1  2003/11/30 16:56:35  Administrator
-    merged into default
-
-    Revision 1.2  2003/11/30 11:44:49  Administrator
-    added CVS id/log
-
+Alfred Reibenschuh.
 
 =cut
