@@ -1,54 +1,22 @@
-#=======================================================================
-#    ____  ____  _____              _    ____ ___   ____
-#   |  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
-#   | |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
-#   |  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
-#   |_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
-#
-#   A Perl Module Chain to faciliate the Creation and Modification
-#   of High-Quality "Portable Document Format (PDF)" Files.
-#
-#   Copyright 1999-2005 Alfred Reibenschuh <areibens@cpan.org>.
-#
-#=======================================================================
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the
-#   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-#   Boston, MA 02111-1307, USA.
-#
-#   $Id: BarCode.pm,v 1.8 2005/03/14 22:01:30 fredo Exp $
-#
-#=======================================================================
 package PDF::API2::Resource::XObject::Form::BarCode;
 
-BEGIN {
+our $VERSION = '2.019';
 
-    use PDF::API2::Util;
-    use PDF::API2::Basic::PDF::Utils;
-    use PDF::API2::Resource::XObject::Form::Hybrid;
+use base 'PDF::API2::Resource::XObject::Form::Hybrid';
 
-    use POSIX;
+use PDF::API2::Util;
+use PDF::API2::Basic::PDF::Utils;
 
-    use vars qw(@ISA $VERSION);
-
-    @ISA = qw( PDF::API2::Resource::XObject::Form::Hybrid );
-
-    ( $VERSION ) = '$Revision: 1.8 $' =~ /Revision: (\S+)\s/; # $Date: 2005/03/14 22:01:30 $
-
-}
 no warnings qw[ deprecated recursion uninitialized ];
 
+=head1 NAME
+
+PDF::API2::Resource::XObject::Form::BarCode
+
+=head1 METHODS
+
+=over
+
 =item $res = PDF::API2::Resource::XObject::Form::BarCode->new $pdf, %opts
 
 Returns a barcode-form object.
@@ -65,13 +33,13 @@ sub new {
 
     $self->{' bfont'}=$opts{-font};
 
-    $self->{' umzn'}=$opts{-umzn};      # (u)pper (m)ending (z)o(n)e
-    $self->{' lmzn'}=$opts{-lmzn};      # (l)ower (m)ending (z)o(n)e
-    $self->{' zone'}=$opts{-zone};
-    $self->{' quzn'}=$opts{-quzn};      # (qu)iet (z)o(n)e
-    $self->{' ofwt'}=$opts{-ofwt}||0.01;        # (o)ver(f)low (w)id(t)h
-    $self->{' fnsz'}=$opts{-fnsz};      # (f)o(n)t(s)i(z)e
-    $self->{' spcr'}=$opts{-spcr}||'';
+    $self->{' umzn'}=$opts{-umzn} || 0;    # (u)pper (m)ending (z)o(n)e
+    $self->{' lmzn'}=$opts{-lmzn} || 0;    # (l)ower (m)ending (z)o(n)e
+    $self->{' zone'}=$opts{-zone} || 0;
+    $self->{' quzn'}=$opts{-quzn} || 0;    # (qu)iet (z)o(n)e
+    $self->{' ofwt'}=$opts{-ofwt} || 0.01; # (o)ver(f)low (w)id(t)h
+    $self->{' fnsz'}=$opts{-fnsz};         # (f)o(n)t(s)i(z)e
+    $self->{' spcr'}=$opts{-spcr} || '';
 
     return($self);
 }
@@ -255,36 +223,10 @@ sub height {
 
 __END__
 
+=back
+
 =head1 AUTHOR
 
-alfred reibenschuh
-
-=head1 HISTORY
-
-    $Log: BarCode.pm,v $
-    Revision 1.8  2005/03/14 22:01:30  fredo
-    upd 2005
-
-    Revision 1.7  2004/12/16 00:30:54  fredo
-    added no warn for recursion
-
-    Revision 1.6  2004/06/15 09:14:54  fredo
-    removed cr+lf
-
-    Revision 1.5  2004/06/07 19:44:44  fredo
-    cleaned out cr+lf for lf
-
-    Revision 1.4  2003/12/08 13:06:09  Administrator
-    corrected to proper licencing statement
-
-    Revision 1.3  2003/11/30 17:34:51  Administrator
-    merged into default
-
-    Revision 1.2.2.1  2003/11/30 16:57:09  Administrator
-    merged into default
-
-    Revision 1.2  2003/11/30 11:50:45  Administrator
-    added CVS id/log
-
+Alfred Reibenschuh
 
 =cut
